/****************************************************************************
 *
 * Copyright (c) 2011-2012, EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA 
 *
 *****************************************************************************/
 
package javax.jbi;

import static org.junit.Assert.fail;

import java.lang.reflect.Method;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Common class for API tests. 
 * <p>
 * Must Be implemented by test that want to test
 * Exceptions that api methods throw.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public abstract class ApiChecker {

    /**
     * Return the api class that is tested
     * 
     * @return
     */
    protected abstract Class<?> getTestedApiClass();

    /**
     * Return a runtime method of of the tested api class. Use reflection
     * 
     * @param methodName
     *            a method name
     * @param parameterTypes
     *            method parameter classes
     * @return a runtime {@link Method}
     */
    protected Method getTestedClassMethod(String methodName, Class<?>... parameterTypes) {
        return ReflectionHelper.getPublicMethod(getTestedApiClass(), methodName, parameterTypes);
    }

    /**
     * Check that Exceptions throwable by the given runtime methods are
     * subclasses of expected exceptions
     * 
     * @param method
     *            a checked Api {@link Method}
     * @param expectedExceptions
     *            expected Exception this method must throw
     */
    protected void checkMethodThrowsExpectedExceptions(Method method, Class<?>[] expectedExceptions) {
        checkRuntimeExceptionsAreSubclassOfExpectedExceptions(method.getExceptionTypes(),
                expectedExceptions);
    }

    /**
     * Check that Exceptions throwable by runtime methods are subclasses of
     * expected exceptions
     * 
     * @param runtimeExceptions
     * @param expectedExceptions
     */
    private void checkRuntimeExceptionsAreSubclassOfExpectedExceptions(
            Class<?>[] runtimeExceptions, Class<?>[] expectedExceptions) {
        for (Class<?> runtimeException : runtimeExceptions) {
            if (!isRuntimeClassSubclassOfExpectedClasses(runtimeException, expectedExceptions)) {
                fail("Exception '" + runtimeException.getSimpleName()
                        + "' isn't a subclass of expected exceptions");
            }
        }
    }

    private boolean isRuntimeClassSubclassOfExpectedClasses(Class<?> runtimeClass,
            Class<?>[] expectedClasses) {
        boolean isSubclass = false;
        for (Class<?> expectedClass : expectedClasses) {
            if (expectedClass.isAssignableFrom(runtimeClass)) {
                isSubclass = true;
                break;
            }
        }
        return isSubclass;
    }

}
